# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import requests

from . client import GeoRapidClient
from . formats import OutFormat



def aggregate(client: GeoRapidClient, date: datetime = datetime.utcnow(), format = OutFormat.GEOJSON):
    """
    Aggregates the broadcasted news related to wildfires using a spatial grid and returns the features as hexagonal bins.
    The date is optional. When not specified, we use datetime.utcnow().
    The underlying knowledge graph collects data from '2015-03-01' up to today.
    The format can be GeoJSON or Esri JSON.
    """
    endpoint = '{0}/aggregate'.format(client.url)
    params = {
        'format': str(format)
    }
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()

def articles(client: GeoRapidClient, date: datetime = datetime.utcnow()):
    """
    Returns a list of broadcasted articles related to wildfires.
    The date is optional. When not specified, we use datetime.utcnow().
    The underlying knowledge graph collects data from '2015-03-01' up to today.
    """
    endpoint = '{0}/articles'.format(client.url)
    params = {}
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()

def query(client: GeoRapidClient, date: datetime = datetime.utcnow(), format = OutFormat.GEOJSON):
    """
    Returns the locations related to wildfires.
    The date is optional. When not specified, we use datetime.utcnow().
    The underlying knowledge graph collects data from '2015-03-01' up to today.
    The format can be GeoJSON or Esri JSON.
    """
    endpoint = '{0}/query'.format(client.url)
    params = {
        'format': str(format)
    }
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()


# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from . client import GeoRapidClient



class EnvironmentClientFactory(object):
    """
    Represents a factory creating client instances using the system environment.
    The environment must offer various variables for authenticating against the web service endpoints.
    """

    @staticmethod
    def create_client():
        """
        Creates a new client using 'x_rapidapi_url', 'x_rapidapi_host' and 'x_rapidapi_key' environment variables.
        Raises a ValueError when these variables are not defined!
        """
        if not 'x_rapidapi_url' in os.environ:
            raise ValueError("'x_rapidapi_url' is not defined in the current environment!")
        if not 'x_rapidapi_host' in os.environ:
            raise ValueError("'x_rapidapi_host' is not defined in the current environment!")
        if not 'x_rapidapi_key' in os.environ:
            raise ValueError("'x_rapidapi_key' is not defined in the current environment!")

        url = os.environ['x_rapidapi_url']
        host = os.environ['x_rapidapi_host']
        key = os.environ['x_rapidapi_key']
        auth_headers = {
            'x-rapidapi-host': host,
            'x-rapidapi-key': key
        }

        return GeoRapidClient(url, auth_headers)

    @staticmethod
    def create_client_with_host(host):
        """
        Creates a new client using 'x_rapidapi_key' enironment variable.
        Raises a ValueError when this variable is not defined!
        """
        if not 'x_rapidapi_key' in os.environ:
            raise ValueError("'x_rapidapi_key' is not defined in the current environment!")

        key = os.environ['x_rapidapi_key']
        auth_headers = {
            'x-rapidapi-host': host,
            'x-rapidapi-key': key
        }

        url = "https://%s" % host
        return GeoRapidClient(url, auth_headers)


# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class GeoRapidClient(object):
    """
    Represents a client accessing the geospatial knowledge API services being hosted at Rapid API.
    """

    def __init__(self, url, auth_headers) -> None:
        """
        Initializes this instance using an url and an authorization header dictionary.
        The dictionary must contain 'x-rapidapi-host' and 'x-rapidapi-host' as keys.
        """
        self._url = url
        if not 'x-rapidapi-host' in auth_headers:
            raise ValueError("'x-rapidapi-host' must be specified in the authorization header!")
        if not 'x-rapidapi-key' in auth_headers:
            raise ValueError("'x-rapidapi-key' must be specified in the authorization header!")

        self._auth_headers = auth_headers

    @property
    def auth_headers(self):
        """Returns the authorization header dictionary."""
        return self._auth_headers

    @property
    def url(self):
        """Returns the endpoint url."""
        return self._url



# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from enum import Enum, unique



@unique
class OutFormat(Enum):
    """Represents the supported output formats."""
    ESRI=0
    GEOJSON=1
    JSON=2

    def __str__(self) -> str:
        if 0 == self.value:
            return 'esri'
        elif 1 == self.value:
            return 'geojson'
        elif 2 == self.value:
            return 'json'
        
        return self.name


# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import requests

from . client import GeoRapidClient
from . formats import OutFormat



def aggregate(client: GeoRapidClient, date: datetime = None, format = OutFormat.GEOJSON):
    """
    Aggregates the broadcasted news related to protests/demonstrations using a spatial grid and returns the features as hexagonal bins.
    The date is optional. When not specified, we return the features of the last 24 hours.
    The underlying hosted feature service saves the last 90 days and yesterday should be the latest available date.
    The format can be GeoJSON or Esri JSON.
    """
    endpoint = '{0}/aggregate'.format(client.url)
    params = {
        'format': str(format)
    }
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()

def articles(client: GeoRapidClient, date: datetime = None):
    """
    Returns a list of broadcasted articles related to protests/demonstrations.
    The date is optional. When not specified, we return the articles of the last 24 hours.
    The underlying web service saves the last 90 days and yesterday should be the latest available date.
    """
    endpoint = '{0}/articles'.format(client.url)
    params = {}
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()

def hotspots(client: GeoRapidClient, date: datetime = None, format = OutFormat.GEOJSON):
    """
    Returns the hotspot locations related to protests/demonstrations.
    The date is optional. When not specified, we return the features of the last 24 hours.
    The underlying hosted feature service saves the last 90 days and yesterday should be the latest available date.
    The format can be GeoJSON or Esri JSON.
    """
    endpoint = '{0}/hotspots'.format(client.url)
    params = {
        'format': str(format)
    }
    if date:
        params['date'] = datetime.strftime(date, '%Y-%m-%d')

    return requests.request('GET', endpoint, headers=client.auth_headers, params=params).json()


# Copyright (C) 2022 Jan Tschada (gisfromscratch@live.de)
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

"""
Install with python setup.py sdist bdist_wheel
"""

with open('README.md', 'r', encoding='utf-8') as readme_file:
    long_description = readme_file.read()

setuptools.setup(
    name='georapid',
    version='0.1a3',
    author='Jan Tschada',
    author_email='gisfromscratch@live.de',
    description='Query broadcasted news worldwide and visualize them using spatial aggregations.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Geospatial-AI-DE/georapid-py',
    packages=['georapid'],
    install_requires=[],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
     ]
 )
